#include "precompile.h"

#include <google/protobuf/message.h>
#include <google/protobuf/descriptor.pb.h>

#include "global.h"
#include "cs_proto.pb.h"

time_t Global::BetweenDays(time_t time1, time_t time2)
{
    return (time1 + g_time_zone*3600)/3600/24 - (time2 + g_time_zone*3600)/3600/24;
}

time_t Global::GetDaySeconds(time_t time, int incdays)
{
    return time_t((time + g_time_zone * 3600)/3600/24 + incdays) * 3600 * 24 - 3600 * g_time_zone;
}

int Global::g_nowtime = time(nullptr);
int Global::g_time_zone = 8;
bool Global::g_shutdown = false;

bool CustomParser(f8::MsgHdr& hdr, google::protobuf::Message* msg)
{
    const google::protobuf::Descriptor* descriptor = msg->GetDescriptor();
    const google::protobuf::Reflection* reflection = msg->GetReflection();

    for (int i = 0; i < descriptor->field_count(); ++i) {
        const google::protobuf::FieldDescriptor* field_desc = descriptor->field(i);
        switch (field_desc->cpp_type()) {
        case google::protobuf::FieldDescriptor::CPPTYPE_STRING:
            {
                int fixed_len = field_desc->options().GetExtension(cs::fixed_len);
                if (field_desc->type() == google::protobuf::FieldDescriptor::TYPE_BYTES && fixed_len > 0) {
                    if ((size_t)hdr.offset + fixed_len > (size_t)hdr.buflen) {
                        abort();
                    }
                    std::string val(&hdr.buf[hdr.offset], fixed_len);
                    reflection->SetString(msg, field_desc, val);
                    hdr.offset += val.size() + 1;
                } else {
                    if ((size_t)hdr.offset > (size_t)hdr.buflen) {
                        abort();
                    }
                    std::string val(&hdr.buf[hdr.offset]);
                    reflection->SetString(msg, field_desc, val);
                    hdr.offset += val.size() + 1;
                }
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT32:
            {
                if (hdr.offset + sizeof(int) > (size_t)hdr.buflen) {
                    abort();
                }
                int* p_val = (int*)&hdr.buf[hdr.offset];
                reflection->SetInt32(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT32:
            {
                if (hdr.offset + sizeof(unsigned int) > (size_t)hdr.buflen) {
                    abort();
                }
                unsigned int* p_val = (unsigned int*)&hdr.buf[hdr.offset];
                reflection->SetUInt32(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT64:
            {
                if (hdr.offset + sizeof(long long) > (size_t)hdr.buflen) {
                    abort();
                }
                long long* p_val = (long long*)&hdr.buf[hdr.offset];
                reflection->SetInt64(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT64:
            {
                if (hdr.offset + sizeof(unsigned long long) > (size_t)hdr.buflen) {
                    abort();
                }
                unsigned long long* p_val = (unsigned long long*)&hdr.buf[hdr.offset];
                reflection->SetUInt64(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_FLOAT:
            {
                if (hdr.offset + sizeof(float) > (size_t)hdr.buflen) {
                    abort();
                }
                float* p_val = (float*)&hdr.buf[hdr.offset];
                reflection->SetFloat(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_DOUBLE:
            {
                if (hdr.offset + sizeof(double) > (size_t)hdr.buflen) {
                    abort();
                }
                double* p_val = (double*)&hdr.buf[hdr.offset];
                reflection->SetFloat(msg, field_desc, *p_val);
                hdr.offset += sizeof(*p_val);
            }
            break;
        default:
            {
                abort();
            }
            break;
        }//end switch
    }
    return true;
}

bool CustomPbSerialize(char* buf, int& offset, int buflen, const google::protobuf::Message* msg)
{
    const google::protobuf::Descriptor* descriptor = msg->GetDescriptor();
    const google::protobuf::Reflection* reflection = msg->GetReflection();

    for (int i = 0; i < descriptor->field_count(); ++i) {
        const google::protobuf::FieldDescriptor* field_desc = descriptor->field(i);
        int fixed_len = field_desc->options().GetExtension(cs::fixed_len);
        switch (field_desc->cpp_type()) {
        case google::protobuf::FieldDescriptor::CPPTYPE_STRING:
            {
                std::string val = reflection->GetString(*msg, field_desc);
                if (field_desc->type() == google::protobuf::FieldDescriptor::TYPE_BYTES && fixed_len > 0) {
                    if (offset + fixed_len > buflen) {
                        abort();
                    }
                    memmove(buf + offset, val.data(), val.size());
                    offset += fixed_len;
                } else {
                    if (offset + val.size() + 1 > (size_t)buflen) {
                        abort();
                    }
                    memmove(buf + offset, val.data(), val.size());
                    offset += val.size() + 1;
                    buf[offset - 1] = '\0';
                }
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT32:
            {
                int val_size = fixed_len == 0 ? 4 : fixed_len;
                if (offset + val_size > buflen) {
                    abort();
                }
                int val = reflection->GetInt32(*msg, field_desc);
                if (val_size == 1) {
                    unsigned char byte_val = val;
                    memmove(buf + offset, &byte_val, val_size);
                } else {
                    memmove(buf + offset, &val, val_size);
                }
                offset += val_size;
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT32:
            {
                if (offset + sizeof(unsigned int) > (size_t)buflen) {
                    abort();
                }
                unsigned int val = reflection->GetUInt32(*msg, field_desc);
                memmove(buf + offset, &val, sizeof(val));
                offset += sizeof(val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT64:
            {
                if (offset + sizeof(long long) > (size_t)buflen) {
                    abort();
                }
                long long val = reflection->GetInt64(*msg, field_desc);
                memmove(buf + offset, &val, sizeof(val));
                offset += sizeof(val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT64:
            {
                if (offset + sizeof(unsigned long long) > (size_t)buflen) {
                    abort();
                }
                unsigned long long val = reflection->GetUInt64(*msg, field_desc);
                memmove(buf + offset, &val, sizeof(val));
                offset += sizeof(val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_FLOAT:
            {
                if (offset + sizeof(float) > (size_t)buflen) {
                    abort();
                }
                float val = reflection->GetFloat(*msg, field_desc);
                memmove(buf + offset, &val, sizeof(val));
                offset += sizeof(val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_DOUBLE:
            {
                if (offset + sizeof(double) > (size_t)buflen) {
                    abort();
                }
                double val = reflection->GetFloat(*msg, field_desc);
                memmove(buf + offset, &val, sizeof(val));
                offset += sizeof(val);
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_MESSAGE:
            {
                if (field_desc->is_repeated()) {
                    int field_size = reflection->FieldSize(*msg, field_desc);
                    assert(fixed_len <= 0 || field_size == field_size);
                    for (int i = 0; i < field_size; ++i) {
                        const google::protobuf::Message* val = &reflection->GetRepeatedMessage(*msg, field_desc, i);
                        CustomPbSerialize(buf, offset, buflen, val);
                    }
                } else {
                    const google::protobuf::Message* val = &reflection->GetMessage(*msg, field_desc);
                    CustomPbSerialize(buf, offset, buflen, val);
                }
            }
            break;
        default:
            {
                abort();
            }
            break;
        }//end switch
    }
    return true;
}

int CustomPbSerializeSize(const google::protobuf::Message* msg)
{
    int byte_size = 0;
    const google::protobuf::Descriptor* descriptor = msg->GetDescriptor();
    const google::protobuf::Reflection* reflection = msg->GetReflection();

    for (int i = 0; i < descriptor->field_count(); ++i) {
        const google::protobuf::FieldDescriptor* field_desc = descriptor->field(i);
        int fixed_len = field_desc->options().GetExtension(cs::fixed_len);
        switch (field_desc->cpp_type()) {
        case google::protobuf::FieldDescriptor::CPPTYPE_STRING:
            {
                std::string val = reflection->GetString(*msg, field_desc);
                if (field_desc->type() == google::protobuf::FieldDescriptor::TYPE_BYTES && fixed_len > 0) {
                    byte_size += fixed_len;
                } else {
                    byte_size += val.size() + 1;
                }
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT32:
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT32:
            {
                byte_size += fixed_len == 0 ? 4 : fixed_len;
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_INT64:
        case google::protobuf::FieldDescriptor::CPPTYPE_UINT64:
            {
                byte_size += fixed_len == 0 ? 8 : fixed_len;
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_FLOAT:
            {
                byte_size += 4;
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_DOUBLE:
            {
                byte_size += 8;
            }
            break;
        case google::protobuf::FieldDescriptor::CPPTYPE_MESSAGE:
            {
                if (field_desc->is_repeated()) {
                    int field_size = reflection->FieldSize(*msg, field_desc);
                    assert(fixed_len <= 0 || field_size == field_size);
                    for (int i = 0; i < field_size; ++i) {
                        const google::protobuf::Message* val = &reflection->GetRepeatedMessage(*msg, field_desc, i);
                        byte_size += CustomPbSerializeSize(val);
                    }
                } else {
                    const google::protobuf::Message* val = &reflection->GetMessage(*msg, field_desc);
                    byte_size += CustomPbSerializeSize(val);
                }
            }
            break;
        default:
            {
                abort();
            }
            break;
        }//end switch
    }
    return byte_size;
}
