#include "precompile.h"

#include "playermgr.h"
#include "cs_proto.pb.h"
#include "dbengine.h"
#include "GCListener.h"
#include "app.h"

enum CharacterFlags
{
    CHARACTER_FLAG_NONE                 = 0x00000000,
    CHARACTER_FLAG_UNK1                 = 0x00000001,
    CHARACTER_FLAG_UNK2                 = 0x00000002,
    CHARACTER_LOCKED_FOR_TRANSFER       = 0x00000004,
    CHARACTER_FLAG_UNK4                 = 0x00000008,
    CHARACTER_FLAG_UNK5                 = 0x00000010,
    CHARACTER_FLAG_UNK6                 = 0x00000020,
    CHARACTER_FLAG_UNK7                 = 0x00000040,
    CHARACTER_FLAG_UNK8                 = 0x00000080,
    CHARACTER_FLAG_UNK9                 = 0x00000100,
    CHARACTER_FLAG_UNK10                = 0x00000200,
    CHARACTER_FLAG_HIDE_HELM            = 0x00000400,
    CHARACTER_FLAG_HIDE_CLOAK           = 0x00000800,
    CHARACTER_FLAG_UNK13                = 0x00001000,
    CHARACTER_FLAG_GHOST                = 0x00002000,
    CHARACTER_FLAG_RENAME               = 0x00004000,
    CHARACTER_FLAG_UNK16                = 0x00008000,
    CHARACTER_FLAG_UNK17                = 0x00010000,
    CHARACTER_FLAG_UNK18                = 0x00020000,
    CHARACTER_FLAG_UNK19                = 0x00040000,
    CHARACTER_FLAG_UNK20                = 0x00080000,
    CHARACTER_FLAG_UNK21                = 0x00100000,
    CHARACTER_FLAG_UNK22                = 0x00200000,
    CHARACTER_FLAG_UNK23                = 0x00400000,
    CHARACTER_FLAG_UNK24                = 0x00800000,
    CHARACTER_FLAG_LOCKED_BY_BILLING    = 0x01000000,
    CHARACTER_FLAG_DECLINED             = 0x02000000,
    CHARACTER_FLAG_UNK27                = 0x04000000,
    CHARACTER_FLAG_UNK28                = 0x08000000,
    CHARACTER_FLAG_UNK29                = 0x10000000,
    CHARACTER_FLAG_UNK30                = 0x20000000,
    CHARACTER_FLAG_UNK31                = 0x40000000,
    CHARACTER_FLAG_UNK32                = 0x80000000
};

void PlayerMgr::Init()
{

}

void PlayerMgr::UnInit()
{

}

void PlayerMgr::_CMAuthSession(f8::MsgHdr& hdr, const cs::CMAuthSession& msg)
{
    std::string digest = msg.digest();
    int ret = DBEngine::Instance()->ExecQuery("SELECT id, gmlevel, sessionkey, last_ip, locked, v, s, mutetime,"
                                              "       locale, os FROM zero_realm.account WHERE username='%s';",
                                              {
                                                  msg.account()
                                              });
    assert(ret > 0);
    GCListener::Instance()->SetKey(hdr.socket_handle,
                                   DBEngine::Instance()->GetValue(2).GetString());
    cs::SMAuthResponse respmsg;
    respmsg.set_errcode(AUTH_OK);
    GCListener::Instance()->SendMsg(hdr.socket_handle, respmsg);
    if (!GetTempSessionBySocket(hdr.socket_handle)) {
        TempSession* tmp_session = new TempSession();
        tmp_session->account_id = DBEngine::Instance()->GetValue(0);
        tmp_session_hash_[hdr.socket_handle] = tmp_session;
    }
}

void PlayerMgr::_CMCharEnum(f8::MsgHdr& hdr, const cs::CMCharEnum& msg)
{
    TempSession* tmp_session = GetTempSessionBySocket(hdr.socket_handle);
    assert(tmp_session);
    std::string sql =
        "SELECT characters.guid, characters.name, characters.race, characters.class, characters.gender, characters.playerBytes, characters.playerBytes2, characters.level, "
        "characters.zone, characters.map, characters.position_x, characters.position_y, characters.position_z, guild_member.guildid, characters.playerFlags, "
        "characters.at_login, character_pet.entry, character_pet.modelid, character_pet.level, characters.equipmentCache "
        "FROM zero_characters.characters LEFT JOIN zero_characters.character_pet ON characters.guid=character_pet.owner AND character_pet.slot='%d' "
        "LEFT JOIN zero_characters.guild_member ON characters.guid = guild_member.guid "
        "WHERE zero_characters.characters.account = '%d' ORDER BY characters.guid";
    int ret = DBEngine::Instance()->ExecQuery(
                                              sql.c_str(),
                                              {
                                                  PET_SAVE_AS_CURRENT,
                                                  tmp_session->account_id
                                              });
    cs::SMCharEnum respmsg;
    if (ret > 0) {
        while (!DBEngine::Instance()->Eof()) {
            auto char_info = respmsg.add_char_list();
            #if 1
            char_info->set_guid(1);
            #else
            char_info->set_guid(App::Instance()->NewUuid());
            #endif
            char_info->set_name(DBEngine::Instance()->GetValue(1).GetString());
            char_info->set_race(DBEngine::Instance()->GetValue(2));
            char_info->set_class_(DBEngine::Instance()->GetValue(3));
            char_info->set_gender(DBEngine::Instance()->GetValue(4));

            unsigned int player_bytes = DBEngine::Instance()->GetValue(5);
            char_info->set_skin((unsigned char)player_bytes);
            char_info->set_face((unsigned char)(player_bytes >> 8));
            char_info->set_hair_style((unsigned char)(player_bytes >> 16));
            char_info->set_hair_color((unsigned char)(player_bytes >> 24));

            unsigned int player_bytes2 = DBEngine::Instance()->GetValue(6);
            char_info->set_facial_hair((unsigned char)player_bytes2);

            char_info->set_level(DBEngine::Instance()->GetValue(7));
            char_info->set_zone(DBEngine::Instance()->GetValue(8));
            char_info->set_map(DBEngine::Instance()->GetValue(9));
            char_info->set_x(DBEngine::Instance()->GetValue(10).GetDouble());
            char_info->set_y(DBEngine::Instance()->GetValue(11).GetDouble());
            char_info->set_z(DBEngine::Instance()->GetValue(12).GetDouble());
            char_info->set_guild_id(DBEngine::Instance()->GetValue(13));

            unsigned int char_flags = 0;
            unsigned int playerFlags = DBEngine::Instance()->GetValue(14);
            unsigned int atLoginFlags = DBEngine::Instance()->GetValue(15);
            if (playerFlags & PLAYER_FLAGS_HIDE_HELM)
                { char_flags |= CHARACTER_FLAG_HIDE_HELM; }
            if (playerFlags & PLAYER_FLAGS_HIDE_CLOAK)
                { char_flags |= CHARACTER_FLAG_HIDE_CLOAK; }
            if (playerFlags & PLAYER_FLAGS_GHOST)
                { char_flags |= CHARACTER_FLAG_GHOST; }
            if (atLoginFlags & AT_LOGIN_RENAME)
                { char_flags |= CHARACTER_FLAG_RENAME; }
            char_info->set_char_flags(char_flags);
            char_info->set_first_login(atLoginFlags & AT_LOGIN_FIRST ? 1 : 0);

            //pet_display_id
            //pet_level
            //pet_family
            for (int i = 0; i < 19; ++i) {
                auto p = char_info->add_inventory_list();
            }
            respmsg.set_char_num(respmsg.char_num() + 1);
            DBEngine::Instance()->Next();
        }
    }
    GCListener::Instance()->SendMsg(hdr.socket_handle, respmsg);
}

int PlayerMgr::OnlineNum()
{
    return 0;
}

Player* PlayerMgr::GetPlayerBySocket(int socket)
{
    return nullptr;
}

void PlayerMgr::OnClientDisconnect(a8::XParams& param)
{
    auto itr = tmp_session_hash_.find(param.sender);
    if (itr != tmp_session_hash_.end()) {
        delete itr->second;
        tmp_session_hash_.erase(itr);
    }
}

void PlayerMgr::RemovePlayerBySocket(int socket_handle)
{

}

TempSession* PlayerMgr::GetTempSessionBySocket(int socket_handle)
{
    auto itr = tmp_session_hash_.find(socket_handle);
    return itr != tmp_session_hash_.end() ? itr->second : nullptr;
}
